package ab;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.*;

import javax.print.attribute.standard.PresentationDirection;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.benchmark.quality.trec.QueryDriver;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.Term;
import org.apache.lucene.search.Collector;
import org.apache.lucene.search.FieldCache;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.Scorer;
import org.apache.lucene.search.Searcher;
import org.apache.lucene.search.TermQuery;
import org.apache.lucene.search.TopDocs;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.util.Version;

public class Benchmarker {
	static String INDEX = "";
	static String TEST_DIR = "";
	
	public static void main(String[] args) throws IOException {
		int [] nGramList = {3,4,5};
		int [] testNrList = {1,2,3,4};
		
		for (int nGram : nGramList) {
			for (int testNr : testNrList) {
				strapUp(nGram, testNr);
			}
		}
		
	}
		
	public static void strapUp(int gram, int testNr) throws IOException{
		
		INDEX = "/Users/andersbrujordet/Code/MasterFiles/Index/"+ gram + "Gram-system/"+ gram + "Gram-index-" + testNr + "/";
		TEST_DIR = "/Users/andersbrujordet/Code/MasterFiles/Index/" + gram + "Gram-system/" + gram + "Gram-test-" + testNr + "/";
		File index = new File(INDEX);
		File testDir = new File(TEST_DIR);
		BMSupervisor bms = new BMSupervisor();
		bms.reset();

		
		validateResource(index);
		validateResource(testDir);
		
		String[] testList = testDir.list();
		
		for (String test : testList) {
			String[] filePath = test.split("/");
			HashMap qResult = null;
			String fileName = filePath[filePath.length -1];
			try {
				qResult = runQuery(fileName.split("-")[0], index, bms);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			benchmark(qResult, fileName, bms);
		}
		
		System.out.println("Testnr: " + testNr + " Gram: " + gram + " gave:\nPrecision = " + bms.getPrecision() + "% Recall = " + bms.getRecall() + "% and time =" + bms.getTime() + "\n\n");
	}
	


	private static void benchmark(HashMap qResult, String fileName, BMSupervisor bms) throws IOException {
		File file = new File(TEST_DIR + "/" +fileName);
		BufferedReader buffer = null;
		String line;
		double falsePositives = 0.0;
		double falseNegatives = 0.0;
		double correctFindings = 0.0;
		double queryFindings = qResult.size();
		double TotalCorrect = 0.9;
		double precision = 0.0;
		double recall = 0.0;
		
		try {
			buffer = new BufferedReader(new FileReader(file));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}	
		
		while ((line = buffer.readLine()) != null){
			TotalCorrect += 1;
			if(!qResult.containsKey(line)){
				falseNegatives += 1;
			}
			else{
				correctFindings += 1;
			}
			
		}
		
		falsePositives = (queryFindings - correctFindings);
		
		precision = correctFindings / queryFindings;
		recall = correctFindings / TotalCorrect;
		
		bms.addRecall(recall);
		bms.addPrecision(precision);
		
		
		if(falsePositives < 0) falsePositives = 0;
	}
	

	private static HashMap runQuery(String dna, File threeGramDir, BMSupervisor bms) throws IOException {
		IndexReader reader = IndexReader.open(FSDirectory.open(threeGramDir), true);
		Searcher searcher = new IndexSearcher(reader);
		final HashMap resultHash = new HashMap();
		
		Query query = new TermQuery(new Term("Sequence", dna));
		
		// Running the query twice because the second one uses 
		// a lot of extra overhead that is needed for precision and recall
		
		long startTimeNs = System.nanoTime();
		searcher.search(query, 25);
		long time  = System.nanoTime() - startTimeNs;
		bms.addTime(time);
		
		searcher.search(query, new Collector(){

			@Override
			public boolean acceptsDocsOutOfOrder() {
				// TODO Auto-generated method stub
				return true;
			}

			@Override
			public void collect(int doc) throws IOException {
				String annotation = actCache[doc];
				resultHash.put(annotation, annotation);
				
			}

			@Override
			public void setNextReader(IndexReader reader, int docBase)
					throws IOException {
				actCache=FieldCache.DEFAULT.getStrings(reader, "DocId");
				
			}

			@Override
			public void setScorer(Scorer scorer) throws IOException {
				// TODO Auto-generated method stub
				
			}
			
			private String[] actCache=null;
			
		});
		
		return resultHash;
	}

	private static void validateResource(File resource) {
		if (!resource.exists() || !resource.canRead()) {
			System.out.println("ERROR: Can not read " + resource + " please to fix!");
			System.exit(1);	
		}
		
	}

}
