package ab;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Date;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.Field.Index;
import org.apache.lucene.document.Field.Store;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.store.FSDirectory;

public class NGramIndexer {

	static String folder;
	static String output;
	static IndexWriter indexWriter;
	static final File INDEX_DIR = new File(
			"/Users/andersbrujordet/Code/MasterFiles/Index/4Gram-system/4Gram-index-2");
	static final String ROOT_DIR = "/Users/andersbrujordet/Code/MasterFiles/Sequences/Fasta-2/";

	public static void main(String[] args) throws IOException {

		if (INDEX_DIR.exists()) {
			System.out.println("Cannot save index to '" + INDEX_DIR
					+ "' directory, please delete it first");
			System.exit(1);
		}

		final File dnaDir = new File(ROOT_DIR);

		if (!dnaDir.exists() || !dnaDir.canRead()) {
			System.out
					.println("Document directory '"
							+ dnaDir.getAbsolutePath()
							+ "' does not exist or is not readable, please check the path");
			System.exit(1);
		}

		Date start = new Date();
		try {
			IndexWriter writer = new IndexWriter(FSDirectory.open(INDEX_DIR),
					new NGramAnalyzer(), true,
					IndexWriter.MaxFieldLength.UNLIMITED);
			System.out.println("Indexing to directory '" + INDEX_DIR + "'...");
			indexDocs(writer, dnaDir);
			System.out.println("Optimizing...");
			writer.optimize();
			writer.close();

			Date end = new Date();
			System.out.println(end.getTime() - start.getTime()
					+ " total milliseconds");
		} catch (IOException e) {
			System.out.println(" caught a " + e.getClass()
					+ "\n with message: " + e.getMessage());
		}

	}

	static void indexDocs(IndexWriter writer, File file) throws IOException {
		if (file.canRead()) {
			if (file.isDirectory()) {
				String[] files = file.list();
				// In case of IO error
				if (files != null) {
					for (int i = 0; i < files.length; i++) {
						indexDocs(writer, new File(file, files[i]));
					}
				}
			} else {
				System.out.println("adding " + file);
				try {
					String sequence = "";
					String annotation = "";
					String docId = "";
					String line = "";
					BufferedReader buffer = new BufferedReader(new FileReader(file));					

					while ((line = buffer.readLine()) != null){
						if(line.contains(">")){

							if(annotation.equals("")){
								annotation = line.replace(">EMBL_CDS:", "");
								docId = annotation.split(" ")[1];
							}
							else{
								Document doc = new Document();
								doc.add(new Field("DocId", docId, Store.YES, Index.NOT_ANALYZED ));
								doc.add(new Field("Annotation", annotation, Store.YES, Index.ANALYZED ));
								doc.add(new Field("Sequence", sequence, Store.NO, Index.ANALYZED ));
								writer.addDocument(doc);
								sequence = "";
								annotation = line.replace(">EMBL_CDS:", "");
								docId = annotation.split(" ")[1];
								}
							
						}
						else{
							sequence += line;
						}
					}
					
					Document doc = new Document();
					doc.add(new Field("DocId", docId, Store.YES, Index.NOT_ANALYZED ));
					doc.add(new Field("Annotation", annotation, Store.YES, Index.ANALYZED ));
					doc.add(new Field("Sequence", sequence, Store.NO, Index.ANALYZED ));
					writer.addDocument(doc);
					
					buffer.close();

				} catch (FileNotFoundException fnfe) {
					System.out.println(fnfe);
				}
			}
		}
	}


}
